clc
clear all
close all

%------------------------input variables-----------------------------------

config=[1 0 1 2 1 0 12 1];%config(1): vertical velocity component due to the waves: 0:off, 1:on
                         %config(2): 0:regular waves, 1:irregular waves
                         %config(3): turbine scale, 1:prototype, 14:model
                         %config(4): number of blades
                         %config(5): vertical velocity profile 0:uniform, %1:no-slip
                         %config(6): tides 0:off, 1:on 
                         %config(7): turbulence intensity TI 0:off, X:X in %
                         %config(8): without waves:0, with waves: 1

fid = fopen('var.txt','r');
c= textscan(fid, '%f %s', 'delimiter','\t');
vari= c{1};                     
V_ini=vari(1);     %input velocity [m/s] - if tides not considered
tmin=vari(2);      %input t start
deltat=vari(3);    %input delta t [s]
tmax=vari(4);      %input t end
steepness=vari(5); %input steepness wave
Twave=vari(6);     %input period wave
fclose(fid);

hydrofoils={'S816-re123.txt', 'S825-re194.txt', 'S826_14re198.txt', 'S826_12re191.txt'};
aero_arr=cell(1, length(hydrofoils));   %empty 1*4 array of matrices

for i=1:1:length(hydrofoils)            %fill the aero_arr with one hydrofoil for each cell:aero_arr{i}
    aero_arr{i}=load(hydrofoils{i});    %loads the variables from a MAT-file into a structure array, aero_arr
end

sf=config(3);       %turbine scale
Nblade=config(4);   %number of blades

%------------------------const param---------------------------------------

N=100;              %100 elementer      
Dt=21/sf;        %turbine diameter [m]- 19.95 for comparison with model tests. 21 m otherwise.
R=Dt/2;             %radius[m]
dr=R/N;             %depth of element [m]       
if config(3)==1     %density of water [kg/m^3]
    rho=1028;
else
    rho=1000; 
end
Ni=7/sf;            %distance from blade tip in lower position to seabed[m]
TSR=7;              %tip speed ratio
TotWD_ini=36/sf;    %total water depth, mean water surface [m]
WDup_ini=(TotWD_ini-Ni-Dt);%depth of turbine blade tip in upper position from mean water surface [m]

%-------------------------read file----------------------------------------

fid = fopen('TIDAL.txt','r');
C = textscan(fid, '%f %f %f %f %f', 'delimiter','\t');
rR = C{1};
r = C{2};
Lc= C{3};
Twist = C{4};
fclose(fid); 

fid = fopen ('a_axial.txt', 'r');
d= textscan(fid, '%f %s', 'delimiter','\t');
a_ax= d{1};
fclose(fid);

fid = fopen ('a_rotational.txt', 'r');
e= textscan(fid, '%f %s', 'delimiter','\t');
a_rot= e{1};
fclose(fid);

%------------------------matrices------------------------------------------

t=tmin:deltat:tmax;              %time array
gamma=zeros(1,Nblade);        %array, angle between horizontal axis and each blade
z=zeros(1,Nblade);            %array, distance betweeen blade element and mean water level
u=zeros(1,Nblade);            %rotational speed
v=zeros(1,Nblade);            %flow velocity
phi=zeros(1,Nblade);          %flow angle
alpha=zeros(1,Nblade);        %angle of attack
ca=zeros(1,Nblade);           %axial induction coefficient
cr=zeros(1,Nblade);           %rotational induction coefficient
Cl=zeros(1,Nblade);           %lift coefficient
Cd=zeros(1,Nblade);           %drag coefficient
dT=zeros(1,Nblade);           %thrust on one blade element for each blade
dM=zeros(1,Nblade);           %torque on one blade element for each blade
T=zeros(Nblade,length(t));    %thrust force each blade, sum all blade elements due to time
Mx=zeros(Nblade,length(t));   %torque each blade, sum all blade elements due to time 
Ttot=zeros(1,length(t));      %thrust all blades due to time
Mtot=zeros(1,length(t));      %torque all blades due to time  
H=zeros(1,length(t));         %water level due to the tides
WaterDepth=zeros(1,length(t));%total water depth [m]
U=zeros(1,length(t));         %water velocity due to the tides
Vel=zeros(1, length(t));      %current velocity [m/s]
u_fluct_ini=zeros(N,length(t));     %intial u fluctuations matrix [N-elements, t-time]
u_fluct=zeros(N,length(t));         %scaled u fluctuations matrix [N-elements, t-time]
Tm=zeros(Nblade, length(t));        %thrust*arm
r_Tm=zeros(Nblade, length(t));      %position where the total thurst force is acting [m]
length_Mz=zeros(Nblade,length(t));  %arm - moment around z-axis 
length_My=zeros(Nblade,length(t));  %arm - moment around x-axis 
My=zeros(Nblade, length(t));        %My, pitch moment expressed at the hub, for each blade due to time 
Mz=zeros(Nblade, length(t));        %Mz, yaw moment expressed at the hub, for each blade due to time 
Mytot=zeros(1,length(t));           %My all blades due to time
Mztot=zeros(1,length(t));           %Mz all blades due to time    
surface_wave=zeros(1, length(t));   %water surface due to the waves
position_blade=zeros(1, length(t)); %position of blade over time

%plot vertical velocity profile
% WD=TotWD_ini:-1:0;                  %array total water depth, one cell for one meter
% vert=zeros(1, length(WD));          %vertical velocity profile at each position of WD, uniform current+waves 
% vertw=zeros(1, length(WD));         %vertical velocity profile at each position of WD, no-slip current+waves
% vertical=cell(1, length(t));        %creates an empty cell array of matrices, vert-matrix in each cell, over time
% verticalw=cell(1, length(t));       %creates an empty cell array of matrices, vertw-matix in each cell, over time

%------------------------turbulence----------------------------------------

if config(7)>0
    for l=1:N
        for k=1:length(t)
            u_fluct_ini(l, k)=turbulence (t(k)); %fluctuations above mean [m/s]
        end
    end
end

%**************************************************************************
%**************************************************************************

%------------------ITERATION T TIMESTEPS ----------------------------------

for j=1:length(t)             %iterating through time array, t
   
%---------------------------tides------------------------------------------

if config(6)==1               %if tides considered
  [H(j),U(j)]=Tides(t(j));
  TotWD=TotWD_ini+H(j);       %total water depth due to the tides[m]  
  WDup=TotWD-Ni-Dt;           %depth of turbine blade tip in upper position due to the tides[m]
  Vin=abs(U(j));              %absolute value - magnitude [m/s]
else                          %else: tides not considered
  WDup=WDup_ini;              %depth of turbine blade tip in upper position [m]
  TotWD=TotWD_ini;            %total water depth [m]
  Vin=V_ini;                  %current velocity [m/s]
end

Vel(j)=Vin;
WaterDepth(j)=TotWD;
    
axisPos=WDup+(Dt/2);          %position of the axis [m] - reference mean water surface

% %---------------------turbulence fluctuations-----------------------

TI=config(7)/100;             %turbulence intensity      
u_f=TI*Vin;                   %fluctuations above mean [m/s]          

%---------------------------turbine control--------------------------------
%for model test: no turbine control 

if Vin < 2.5
   omegat=TSR*Vin/R;         %angular frequency turbine [rad/s]
    % Tturbine=2*pi/omegat; %period turbine [s]
 else
   omegat=TSR*2.5/R;         %angular frequency turbine [rad/s]
    %Tturbine=2*pi/omegat; %period turbine [s]
end

%----------------------------waves-----------------------------------------

lambdaw=(9.81/(2*pi))*Twave^2;%wave length [m]
Hwave=lambdaw*steepness;      %wave height max[m] 

k_gen=2*pi/lambdaw;           %wave number generated [rad/m]
omegaw_gen=2*pi/Twave;        %wave frequency generated [rad/s]

omegaw=omegaw_gen+k_gen*Vin;  %frequency of encounter [rad/s]
k=(omegaw^2)/9.81;            %wave number of encounter [rad/m]   
Twave_encounter=2*pi/omegaw;  %wave period of encounter [s]

Vwmax=(Hwave/2)*omegaw;       %water particle velocity at the surface [m/s]

surface_wave(j)=Hwave/2*cos(-(omegaw*t(j)));%surface wave form --> plot 

%------------------plot vertical velocity profile--------------------------

% [vertw vert]=profile(TotWD_ini, axisPos, k, Vwmax, omegaw, t(j), Vin, Hwave, config(5), WD);
% verticalw{j}=vertw;
% vertical{j}=vert;
% 
%---------------------------gamma------------------------------------------

    gamma_ini=omegat*t(j);    %angle between blade and horisontal axis, reference horisontal axis z=Dt/2+WDup=18.5 meters below sea level 
                              %positiv direction left
    
    position_blade(j)=sin(gamma_ini);%position blade --> plot                          

%--------------------ITERATION N ELEMENTS----------------------------------
    
    for i=1:1:N               %iterating through N elements 
       
%-----------------------------tiploss--------------------------------------

        F=tiploss(i);              

%-----------------------turbulence blade element---------------------------
% %Vin is velocity on axis
%         
%         if config(7)>0                             %if turbulence considered   
%             if Vin>0                               %if current velocity not 0            
%                 a=(u_f/std(u_fluct_ini(i, :)));    %scaling factor for u_fluct_ini
%                 u_fluct(i, j)=a.*u_fluct_ini(i, j);%scaling initial u'
%                 V=Vin+u_fluct(i, j);               %current velocity = u_mean + u'
%             else                                   %else current velocity is 0 
%                 V=0;
%             end
%         else                                       %else turbulence not considered
%             V=Vin;
%         end
%--------------------ITERATION Nblade BLADES-------------------------------
  
        for index_blade=1:Nblade    %iterating through n blades
            
%-------------------------position blade-----------------------------------
            
            gamma(index_blade)=gamma_ini+(2*pi)/Nblade*(index_blade-1);%angle to horizontal axis
            z(index_blade)=Dt/2+WDup-r(i)*sin(gamma(index_blade));     %blade element position on vertical axis from sea surface, y(surface)=0
                                                                   
%------------------------velocities blade----------------------------------
            
            [v(index_blade) u(index_blade)]= vel(z(index_blade), TotWD, axisPos, k, Vwmax, omegaw, t(j), Vin, Hwave, config(5), config(1), r(i), omegat, gamma(index_blade), config(8));        

%------------------------------turbulence----------------------------------      
        
        if config(7)>0                                      %if turbulence considered   
                a=(u_f/std(u_fluct_ini(i, :)));             %scaling factor for u_fluct_ini
                u_fluct(i, j)=a.*u_fluct_ini(i, j);         %scaling initial u'
                v(index_blade)=v(index_blade)+u_fluct(i, j);%current velocity = u_mean + u'
        end  
      
%---------------------------angles blade-----------------------------------
%angles in degrees from list
 
            phi(index_blade)=atand((1-a_ax(i))*v(index_blade)/((1+a_rot(i))*u(index_blade)));
 
            theta=Twist(i);                                                                   %twist angle
            alpha(index_blade)=phi(index_blade)-theta;                                        %angle of attack
        
%------------------------force coefficients blade--------------------------

            [Cl(index_blade) Cd(index_blade)]=LiftDragCoeff(alpha(index_blade), rR(i), aero_arr);

            ca(index_blade)=Cl(index_blade)*cosd(phi(index_blade))+Cd(index_blade)*sind(phi(index_blade)); %axial coefficient
            cr(index_blade)=Cl(index_blade)*sind(phi(index_blade))-Cd(index_blade)*cosd(phi(index_blade)); %rotational coeffecient

%---------------------------forces blade ----------------------------------

            dT(index_blade)=ca(index_blade)*0.5*rho*Lc(i)*dr*v(index_blade)*abs(v(index_blade))*(1-a_ax(i))^2/(sind(phi(index_blade))*abs(sind(phi(index_blade))))*F; %Thrust force one element on one blade
            dM(index_blade)=cr(index_blade)*0.5*rho*Lc(i)*dr*v(index_blade)*(1-a_ax(i))/(sind(phi(index_blade)))*omegat*r(i)*(1+a_rot(i))/(cosd(phi(index_blade)))*F; %Moment one element on one blade 
            
            T(index_blade, j)=T(index_blade, j)+dT(index_blade);        %Thrust force each blade, all blade elements 
            Mx(index_blade, j)=Mx(index_blade, j)+dM(index_blade)*r(i); %Torque each blade, all blade elements
            Tm(index_blade, j)=Tm(index_blade, j)+dT(index_blade)*r(i); %Sum: Thrust each blade element*radius
            
        end %end blades
        
    end %end blade elements
%--------------------------------------------------------------------------

    for index_blade=1:Nblade
        r_Tm(index_blade,j)=Tm(index_blade,j)/T(index_blade, j);              %Radius, where, on the blade, the thrust force is acting
        length_Mz(index_blade,j)=cos(gamma_ini+(2*pi)/Nblade*(index_blade-1))*r_Tm(index_blade,j); %distance where the force is acting from the z-axis
        length_My(index_blade,j)=sin(gamma_ini+(2*pi)/Nblade*(index_blade-1))*r_Tm(index_blade,j); %distance where the force is acting from the y-axis
%         Mz(index_blade, j)=Mz(index_blade, j)+T(index_blade, j)*length_Mz(index_blade, j);   %moment around z-axis
%         My(index_blade, j)=My(index_blade, j)+T(index_blade, j)*length_My(index_blade, j);   %moment around y-axis
       Mz(index_blade,j)=T(index_blade,j)*length_Mz(index_blade,j);   %moment around z-axis
       My(index_blade,j)=T(index_blade,j)*length_My(index_blade,j);   %moment around x-axis
    end

    Ttot(j)=sum(T(:,j));    %total thrust force as a function of time
    Mtot(j)=sum(Mx(:,j));   %total torque as a function of time
  
    Mytot(j)=sum(My(:,j));        
    Mztot(j)=sum(Mz(:,j));
    
end %end timestep

T1=T(1,:);
T2=T(2,:);

Mx1=Mx(1,:);
Mx2=Mx(2,:);

My1=My(1,:);
My2=My(2,:);

Mz1=Mz(1,:);
Mz2=Mz(2,:);

meanThrust1=mean(T1)
minThrust1=min(T1)
maxThrust1=max(T1)
stdThrust1=std(T1)

meanThrust=mean(Ttot)
minThrust=min(Ttot)
maxThrust=max(Ttot)
stdThrust=std(Ttot)

meanTorque1=mean(Mx1)
minTorque1=min(Mx1)
maxTorque1=max(Mx1)
stdTorque1=std(Mx1)

meanShaftTorque=mean(Mtot)
minShaftTorque=min(Mtot)
maxShaftTorque=max(Mtot)
stdShaftTorque=std(Mtot)

meanMy1=mean(My1)
minMy1=min(My1)
maxMy1=max(My1)
stdMy1=std(My1)

meanMytot=mean(Mytot)
minMytot=min(Mytot)
maxMytot=max(Mytot)
stdMytot=std(Mytot)

meanMz1=mean(Mz1)
minMz1=min(Mz1)
maxMz1=max(Mz1)
stdMz1=std(Mz1)

meanMztot=mean(Mztot)
minMztot=min(Mztot)
maxMztot=max(Mztot)
stdMztot=std(Mztot)

%-------------------------------plot---------------------------------------
% 
%plot thrust each blade and total 
 figure(1) 
 hold on
 plot(t, T1,'-k')
 plot(t, T2,'-g')
 plot(t, Ttot,'-b')
 %legend('Thrust blade 1', 'Thrust blade 2','Location','NorthWest');
 %hold off
 xlabel('Time [s]','fontsize',12)
 ylabel('Thrust [N]' ,'fontsize',12)
 %title({['Thrust force each blade with a a wave period of ' num2str(Twave) ' s and steepness of ' num2str(steepness)]},'fontsize',12)
 grid on
 box on 

% %plot thrust tot
%  figure(2)
%  plot(t, Ttot,'-b')
%  %legend('Thrust','Location','NorthWest')
%  hold off
%  xlabel('Time [s]','fontsize',12)
%  ylabel('Tot. Thrust [N]' ,'fontsize',12)
% % title({['Total thrust force with a a wave period of ' num2str(Twave) ' s and steepness of ' num2str(steepness)]},'fontsize',12)
%  grid on
%  box on
%  
 %plot moment each blade and total 
 figure(3)
 hold on
 plot(t, Mx1,'-k')
 plot(t, Mx2,'-g')
 plot(t, Mtot,'-b')
 %legend('Torque blade 1', 'Torque blade 2','Location','NorthWest');
 hold off
 xlabel('Time [s]','fontsize',12)
 ylabel('Torque [Nm]' ,'fontsize',12)
% title({['Torque each blade with a a wave period of ' num2str(Twave) ' s and steepness of ' num2str(steepness)]},'fontsize',12)
 grid on
 box on
%  
% %plot moment tot
% figure(4)
%  plot(t, Mtot,'-g')
%  %legend('Torque blade 1','Location','NorthWest')
%  xlabel('Time [s]','fontsize',12)
%  ylabel('Tot. Torque [Nm]' ,'fontsize',12)
%  %title({['Total torque with a a wave period of ' num2str(Twave) ' s and steepness of ' num2str(steepness)]},'fontsize',12)
%  grid on
%  box on
 
% %plot tot thrust and moment
%  figure(5)
%  hold on
%  plot(t, Ttot,'-b')
%  plot(t, Mtot,'-g')
%  legend('Thrust','Moment','Location','NorthWest')
%  hold off
%  xlabel('Time [s]','fontsize',12)
%  ylabel('Thrust [N], Torque [Nm]' ,'fontsize',12)
%  %title({['Total thrust and torque with a a wave period of ' num2str(Twave) ' s and steepness of ' num2str(steepness)]},'fontsize',12)
%  grid on
%  box on

%plot My each blade and My_tot
 figure(6)
 hold on
 plot(t, My1, '-k')
 plot(t, My2, '-g')
 plot(t, Mytot,'-b')
 %legend('My blade 1','My blade 2','My tot','Location','NorthWest')
 hold off
 xlabel('Time [s]','fontsize',12)
 ylabel('My [Nm]' ,'fontsize',12)
 grid on
 box on
 
 %plot Mz each blade and Mz_tot
 figure(7)
 hold on
 plot(t, Mz1, '-k')
 plot(t, Mz2, '-g')
 plot(t, Mztot,'-b')
 %legend('Mz blade 1','Mz blade 2','Mz tot','Location','NorthWest')
 hold off
 xlabel('Time [s]','fontsize',12)
 ylabel('Mz [Nm]' ,'fontsize',12)
 grid on
 box on
 
 %plot U og H tides
 if config(6)== 1
 figure(8)
 hold on
 plot(t, H,'-b')
 plot(t, abs(U),'-g')
 hold off
 xlabel('Time [s]','fontsize',12)
 ylabel('H [m], U [m/s]' ,'fontsize',12)
 legend('H: Water level','U: Water velocity','Location','NorthWest')
 grid on
 box on
 end
 
 %plot fluctuating velocity
 if config(7)>0
 figure(9)
 hold on
 plot(t, u_fluct_ini (1, :),'-g')
 plot(t, u_fluct(1, :),'-b')
 %plot(t, u_fluct_ini (2, :),'-c')
 %plot(t, u_fluct(2, :),'-m')
 hold off
 xlabel('Time [s]','fontsize',12)
 ylabel('u'' [m/s]' ,'fontsize',12)
 %legend('u-fluct-ini','u-fluct','Location','NorthWest')
 grid on
 box on
 end
 
% %plot surface wave
%  figure(10)
%  hold on
%  plot(t, surface_wave, '-b')
%  plot(t, position_blade, '-k')
%  hold off
%  xlabel('Time [s]','fontsize',12)
%  ylabel('Surface wave, eta, and position of the blade, sin(gamma)' ,'fontsize',12)
%  legend('surface wave','position blade','Location','NorthWest')
%  grid on
%  box on

%plot vertical velocity profile 
%  figure(11)
%  hold on
% % xlim([0 5])
%  plot(verticalw{1}, WD, '-b')           %plot vertical velocity profile: no-slip current+waves {1.timestep,t=0}
%  plot(vertical{1}, WD, '-g')            %plot vertical velocity profile: uniform current+waves {1.timestep,t=0}
%  hold off
%  xlabel('Velocity [m/s]' ,'fontsize',12)
%  ylabel('TotWD[m]','fontsize',12)
%  grid on
%  box on
%  
%  figure(12)
%  hold on
%  xlim([0 3])
%  plot(verticalw{28}, WD, '-b')           %plot vertical velocity profile: no-slip current+waves {1.timestep,t=0}
%  plot(vertical{28}, WD, '-g')            %plot vertical velocity profile: uniform current+waves {1.timestep,t=0}
%  hold off
%  xlabel('Velocity [m/s]' ,'fontsize',12)
%  ylabel('TotWD[m]','fontsize',12)
%  grid on
%  box on
%  
%  figure(13)
%  hold on
%  plot(vertical{1}, WD, '-b')            %plot vertical velocity profile: uniform current+waves {1.timestep,t=0}
%  hold off
%  xlabel('Velocity [m/s]' ,'fontsize',12)
%  ylabel('TotWD[m]','fontsize',12)
%  grid on
%  box on
%--------------------------------------------------------------------------

%save matrices as cell array
% 
% data_1_4=cell(2, 14);
% 
% data_1_4{1,2}='T one blade';
% data_1_4{1,3}='Ttot';
% data_1_4{1,4}='Mx one blade';
% data_1_4{1,5}='Mtot';
% data_1_4{1,6}='My one blade';
% data_1_4{1,7}='Mytot';
% data_1_4{1,8}='Mz one blade';
% data_1_4{1,9}='Mztot';
% data_1_4{1,10}='U';
% data_1_4{1,11}='WaterDepth';
% data_1_4{1,12}='Surface wave';
% data_1_4{1,13}='Position blade';
% data_1_4{1,14}='t';
% 
% data_1_4{2,2}=T1;
% data_1_4{2,3}=Ttot;
% data_1_4{2,4}=Mx1;
% data_1_4{2,5}=Mtot;
% data_1_4{2,6}=My1;
% data_1_4{2,7}=Mytot;
% data_1_4{2,8}=Mz1;
% data_1_4{2,9}=Mztot;
% data_1_4{2,10}=Vel;
% data_1_4{2,11}=WaterDepth;
% data_1_4{2,12}=surface_wave;
% data_1_4{2,13}=position_blade;
% data_1_4{2,14}=t;
% 
% save data_1_4
% save data_1_4_rf.mat T1 Ttot Mx1 Mtot My1 Mytot Mz1 Mztot